@@ -66,15 +66,15 @@ pump_mdes <- function(
         rho = NULL, rho.matrix = NULL,
         B = 1000,
         max.steps = 20, 
-        tnum = 1000, start.tnum = tnum / 10, final.tnum = 4*tnum,
+        tnum = 1000, start.tnum = round( tnum / 10 ), final.tnum = 4*tnum,
         parallel.WY.cores = 1,
         updateProgress = NULL, give.optimizer.warnings = FALSE,
         verbose = FALSE ) {
     
     if ( verbose ) {
-        scat( "pump_mdes with %d max iterations per search, 
-          starting at %d iterations with final %d iterations 
-          (%d perms for WY if used)\n",
+        scat( paste("pump_mdes with %d max iterations per search, starting",
+                    "at %d iterations with final %d iterations",
+                    "(%d perms for WY if used)\n"),
               start.tnum, tnum, final.tnum, B )
     }
     
@@ -95,10 +95,11 @@ pump_mdes <- function(
     tnum_est <- round( (target.power*(1 - target.power) ) / ( tol^2 ) )
     if ( err > tol ) {
         warning( sprintf( 
-            "Number of replicates (tnum) is too small given target tolerance.  Increasing tnum from %d to %d",
+            paste("Number of replicates (tnum) is too small given target",
+                  "tolerance. Increasing tnum from %d to %d"),
             as.integer( tnum ), as.integer( tnum_est ) ), call. = FALSE )
         tnum <- tnum_est
-        start.tnum <- tnum / 10
+        start.tnum <- round( tnum / 10 )
         final.tnum <- 4*tnum
     }
     
@@ -111,7 +112,8 @@ pump_mdes <- function(
         MTP = MTP, numZero = numZero, propZero = propZero,
         M = M, J = J, K = K,
         nbar = nbar, Tbar = Tbar, alpha = alpha, two.tailed = two.tailed,
-        numCovar.1 = numCovar.1, numCovar.2 = numCovar.2, numCovar.3 = numCovar.3,
+        numCovar.1 = numCovar.1, numCovar.2 = numCovar.2, 
+        numCovar.3 = numCovar.3,
         R2.1 = R2.1, R2.2 = R2.2, R2.3 = R2.3,
         ICC.2 = ICC.2, ICC.3 = ICC.3, omega.2 = omega.2, omega.3 = omega.3,
         rho = rho, rho.matrix = rho.matrix, B = B,
@@ -120,9 +122,8 @@ pump_mdes <- function(
         power.definition = power.definition
     )
     ##
-    params.list <- validate_inputs(
-        d_m, params.list, mdes.call = TRUE, verbose = verbose 
-    )
+    params.list <- validate_inputs( d_m, params.list, mdes.call = TRUE, 
+                                    verbose = verbose )
     ##
     MTP <- params.list$MTP
     MDES <- params.list$MDES; numZero <- params.list$numZero;
@@ -131,13 +132,15 @@ pump_mdes <- function(
     alpha <- params.list$alpha; two.tailed <- params.list$two.tailed
     numCovar.1 <- params.list$numCovar.1; numCovar.2 <- params.list$numCovar.2
     numCovar.3 <- params.list$numCovar.3
-    R2.1 <- params.list$R2.1; R2.2 <- params.list$R2.2; R2.3 <- params.list$R2.3
+    R2.1 <- params.list$R2.1; R2.2 <- params.list$R2.2
+    R2.3 <- params.list$R2.3
     ICC.2 <- params.list$ICC.2; ICC.3 <- params.list$ICC.3
     omega.2 <- params.list$omega.2; omega.3 <- params.list$omega.3
     rho <- params.list$rho; rho.matrix <- params.list$rho.matrix
     B <- params.list$B
     power.definition <- params.list$power.definition
-
+    d_m = params.list$d_m
+    
     params.list <- params.list[names(params.list) != 'power.definition']
     
     # extract power definition
@@ -147,7 +150,7 @@ pump_mdes <- function(
     mdes.cols <- c("MTP", "Adjusted.MDES", paste(power.definition, "power"))
     
     # check if zero power, then return 0 MDES
-    if(round(target.power, 2) <= 0)
+    if (round(target.power, 2) <= 0)
     {
         message('Target power of 0 (or negative) requested')
         mdes.results <- data.frame(MTP, 0, 0)
@@ -160,7 +163,7 @@ pump_mdes <- function(
     }
     
     # check if max power, then return infinite MDES
-    if(round(target.power, 2) >= 1)
+    if (round(target.power, 2) >= 1)
     {
         message('Target power of 1 (or larger) requested')
         mdes.results <- data.frame(MTP, Inf, 1)
@@ -173,7 +176,8 @@ pump_mdes <- function(
     }
     
     if ( verbose ) {
-        message(paste("Estimating MDES for", MTP, "for target", power.definition,
+        message(paste("Estimating MDES for", MTP, "for target", 
+                      power.definition,
                       "power of", round(target.power, 4)))
     }
     
@@ -185,18 +189,19 @@ pump_mdes <- function(
     )
     t.df <- calc_df(
         d_m = d_m, J = J, K = K, nbar = nbar,
-        numCovar.1 = numCovar.1, numCovar.2 = numCovar.2, numCovar.3 = numCovar.3
+        numCovar.1 = numCovar.1, numCovar.2 = numCovar.2, 
+        numCovar.3 = numCovar.3
     )
     
     # For raw and BF, compute critical values
-    if(two.tailed)
+    if (two.tailed)
     {
-        crit.alpha <- stats::qt(p = (1-alpha/2), df = t.df)
-        crit.alphaxM <- stats::qt(p = (1-alpha/(2*M)), df = t.df)
+        crit.alpha <- stats::qt(p = (1 - alpha / 2), df = t.df)
+        crit.alphaxM <- stats::qt(p = (1 - alpha / (2 * M)), df = t.df)
     } else
     {
-        crit.alpha <- stats::qt(p = (1-alpha), df = t.df)
-        crit.alphaxM <- stats::qt(p = (1-alpha/M), df = t.df)
+        crit.alpha <- stats::qt(p = (1 - alpha), df = t.df)
+        crit.alphaxM <- stats::qt(p = (1 - alpha / M), df = t.df)
     }
     
     
@@ -204,7 +209,7 @@ pump_mdes <- function(
     crit.beta <- ifelse(target.power > 0.5,
                         stats::qt(target.power, df = t.df),
                         stats::qt(1 - target.power, df = t.df))
-    if(target.power > 0.5)
+    if (target.power > 0.5)
     {
         mdes.raw.list <- Q.m * (crit.alpha + crit.beta)
         mdes.bf.list  <- Q.m * (crit.alphaxM + crit.beta)
@@ -238,7 +243,7 @@ pump_mdes <- function(
     
     # adjust bounds to capture needed range for minimum or complete power.
     # bounds note: complete power is a special case of minimum power
-    if(pdef$min)
+    if (pdef$min)
     {
         # complete power will have a higher upper bound
         # must detect all individual outcomes
@@ -265,12 +270,12 @@ pump_mdes <- function(
     }
     
     # unlikely, but just in case
-    if(mdes.high < 0)
+    if (mdes.high < 0)
     {
         mdes.high <- 0
     }
     # corner case
-    if(mdes.low < 0)
+    if (mdes.low < 0)
     {
         mdes.low <- 0
     }
@@ -303,13 +308,13 @@ pump_mdes <- function(
     )
     colnames(mdes.results) <- mdes.cols
     
-    if(!is.na(mdes.results$`Adjusted.MDES`) && 
+    if (!is.na(mdes.results$`Adjusted.MDES`) && 
        test.pts$dx[[nrow(test.pts)]] < 0.001 )
     {
-        msg <- "Note: this function returns one possible value of MDES, 
-    but other (smaller values) may also be valid.\n"
-        msg <- paste0(msg, "Please refer to sample size vignette for 
-                 interpretation.\n")
+        msg <- paste("Note: this function returns one possible value of",
+                     "MDES, but other (smaller values) may also be valid.\n")
+        msg <- paste(msg, "Please refer to sample size vignette",
+                     "for interpretation.\n")
         message(msg)
         flat <- TRUE
     } else
