@@ -2,6 +2,56 @@
 # generate simulation data
 # ------------------------------#
 
+
+
+process_and_generate_param_list <- function(
+        d_m, param.list, pump.object, Tbar = NULL, 
+        include_Tx = TRUE ) {
+    
+    # If first argument is a pump.object, swap arguments
+    if ( !is.null(d_m) && (is.pumpresult(d_m) || is.pumpgridresult(d_m)) ) {
+        pump.object <- d_m
+        d_m <- NULL
+    }
+    
+    if (is.null(pump.object)) {
+        if ( include_Tx && (is.null(d_m) || is.null(param.list)) ) {
+            stop(paste("You must provide either a pump object or both",
+                    "a design string (d_m) and list of model params."))
+        }
+    } else {
+        if ( !is.null(d_m) || !is.null(param.list) ) {
+            stop(paste("You must provide either a pump object",
+                       "or a design string (d_m) and list of model params",
+                       "pair (not both)."))
+        }
+        
+        param.list <- params(pump.object)
+        param.list$rho.default <- param.list$rho
+        
+        d_m <- d_m(pump.object)
+
+        # Convert pump object to param list
+        if ( include_Tx ) {
+            Tbar <- param.list$Tbar
+        }
+        
+        if ( pump_type(pump.object) == "sample" ) {
+            param.list$nbar = 10 
+            param.list[ pump.object$Sample.type ] = pump.object$Sample.size
+        }
+    }
+    
+    param.list$d_m = d_m
+    if ( include_Tx && !is.null( Tbar ) ) {
+        param.list$Tbar = Tbar
+    }
+    return( param.list )
+}
+
+
+
+
 #' @title Generate correlation matrix (simulation function)
 #'
 #' @param M scalar; dimension of matrix.
@@ -30,8 +80,8 @@ gen_corr_matrix <- function(M, rho.scalar)
 #' @keywords internal
 gen_cov_matrix <- function(D, var1.vec, var2.vec, rho.matrix) {
     Sigma <- matrix(NA, D, D)
-    for(k in 1:D) {
-        for(l in 1:D) {
+    for (k in 1:D) {
+        for (l in 1:D) {
             Sigma[k,l] <- rho.matrix[k,l] * 
                 sqrt(var1.vec[k]) * sqrt(var2.vec[l])
         }
@@ -47,7 +97,7 @@ gen_cov_matrix <- function(D, var1.vec, var2.vec, rho.matrix) {
 #' 
 #' @return 2M x 2M matrix for generating correlated pairs of random effects
 #' @keywords internal
-gen_RE_cov_matrix <- function( Sigma.w, Sigma.z, Sigma.wz ) {
+gen_RE_cov_matrix <- function(Sigma.w, Sigma.z, Sigma.wz) {
     stopifnot( nrow(Sigma.w) == ncol(Sigma.w) )
     stopifnot( all( dim(Sigma.z) == dim( Sigma.wz ) ) )
     stopifnot( all( dim(Sigma.w) == dim( Sigma.wz ) ) )
@@ -55,11 +105,11 @@ gen_RE_cov_matrix <- function( Sigma.w, Sigma.z, Sigma.wz ) {
     M <- nrow(Sigma.w)
     
     # full covariance matrix
-    Sigma.wz.full                            <- matrix(NA, 2*M, 2*M)
-    Sigma.wz.full[1:M, 1:M]                  <- Sigma.w
-    Sigma.wz.full[(M+1):(2*M), (M+1):(2*M)]  <- Sigma.z
-    Sigma.wz.full[1:M, (M+1):(2*M)]          <- Sigma.wz
-    Sigma.wz.full[(M+1):(2*M), 1:M]          <- t(Sigma.wz)
+    Sigma.wz.full                                    <- matrix(NA, 2 * M, 2 * M)
+    Sigma.wz.full[1:M, 1:M]                          <- Sigma.w
+    Sigma.wz.full[(M + 1):(2 * M), (M + 1):(2 * M)]  <- Sigma.z
+    Sigma.wz.full[1:M, (M + 1):(2 * M)]              <- Sigma.wz
+    Sigma.wz.full[(M + 1):(2 * M), 1:M]              <- t(Sigma.wz)
     
     Sigma.wz.full
 }
@@ -73,48 +123,68 @@ gen_RE_cov_matrix <- function( Sigma.w, Sigma.z, Sigma.wz ) {
 #' @description Generates simulated data for multi-level RCTs for
 #'   pump-supported designs and models for both unobserved potential
 #'   outcomes. This function does not generate treatment assignments
-#'   or observed outcomes.
+#'   or observed outcomes--see gen_sim_data() for that.
 #'
-#'   Takes in a list of necessary data-generating parameters.
+#'   This method takes in a list of necessary data-generating
+#'   parameters, following the rest of the package.
 #'
 #'   This function is beyond the main scope of calculating power, and
 #'   is instead used for simulating data. For more info on use, see
 #'   the simulation vignette.
 #'
-#'
-#' @param dgp.params.list list of data generating parameters.
+#' @inheritParams gen_sim_data
+#' @param dgp.params TRUE means param.list is already converted to DGP
+#'   parameters, FALSE means it needs to be converted via
+#'   `convert_params()`.
+#' @seealso gen_sim_data
 #'
 #' @return list; potential outcomes given control y0, treatment y1,
-#'   covariates V.k, X.jk, C.ijk.
+#'   covariates V.k, X.jk, C.ijk, or list of dataframes if
+#'   return.as.dataframe = TRUE.
 #'
 #' @export
-gen_base_sim_data <- function(dgp.params.list) {
+gen_base_sim_data <- function(param.list, pump.object = NULL,
+                              return.as.dataframe = TRUE, 
+                              no.list = TRUE,
+                              dgp.params = FALSE ) {
+    
+    if ( !dgp.params ) {
+        param.list = process_and_generate_param_list( 
+            d_m = NULL, param.list = param.list, Tbar = NULL,
+            pump.object = pump.object, include_Tx = FALSE 
+        )
+    
+        param.list = convert_params( param.list )   
+    }
     
     # ------------------------------#
     # setup: convert model params.list to variables
     # ------------------------------#
-    has.level.three <- dgp.params.list[['has.level.three']]
-    M        <- dgp.params.list[['M']];       J       <- dgp.params.list[['J']]
-    K        <- dgp.params.list[['K']];       nbar    <- dgp.params.list[['nbar']]
-    S.id     <- dgp.params.list[['S.id']];    D.id    <- dgp.params.list[['D.id']]
-    Xi0      <- dgp.params.list[['Xi0']];     Xi1     <- dgp.params.list[['Xi1']]
-    rho.V    <- dgp.params.list[['rho.V']];   xi      <- dgp.params.list[['xi']]
-    eta0.sq  <- dgp.params.list[['eta0.sq']]; eta1.sq <- dgp.params.list[['eta1.sq']]
-    rho.w0   <- dgp.params.list[['rho.w0']];  rho.w1  <- dgp.params.list[['rho.w1']]
-    kappa.w  <- dgp.params.list[['kappa.w']]
-    rho.X    <- dgp.params.list[['rho.X']];   delta   <- dgp.params.list[['delta']]
-    tau0.sq  <- dgp.params.list[['tau0.sq']]; tau1.sq <- dgp.params.list[['tau1.sq']]
-    rho.u0   <- dgp.params.list[['rho.u0']];  rho.u1  <- dgp.params.list[['rho.u1']]
-    kappa.u  <- dgp.params.list[['kappa.u']]
-    rho.C    <- dgp.params.list[['rho.C']];   gamma   <- dgp.params.list[['gamma']]
-    rho.r    <- dgp.params.list[['rho.r']]
+    has.level.three <- param.list[['has.level.three']]
+    M        <- param.list[['M']];       J       <- param.list[['J']]
+    K        <- param.list[['K']];       nbar    <- param.list[['nbar']]
+    S.id     <- param.list[['S.id']];    D.id    <- param.list[['D.id']]
+    Xi0      <- param.list[['Xi0']];     Xi1     <- param.list[['Xi1']]
+    rho.V    <- param.list[['rho.V']];   xi      <- param.list[['xi']]
+    eta0.sq  <- param.list[['eta0.sq']]; eta1.sq <- param.list[['eta1.sq']]
+    rho.w0   <- param.list[['rho.w0']];  rho.w1  <- param.list[['rho.w1']]
+    kappa.w  <- param.list[['kappa.w']]
+    rho.X    <- param.list[['rho.X']];   delta   <- param.list[['delta']]
+    tau0.sq  <- param.list[['tau0.sq']]; tau1.sq <- param.list[['tau1.sq']]
+    rho.u0   <- param.list[['rho.u0']];  rho.u1  <- param.list[['rho.u1']]
+    kappa.u  <- param.list[['kappa.u']]
+    rho.C    <- param.list[['rho.C']];   gamma   <- param.list[['gamma']]
+    rho.r    <- param.list[['rho.r']]
     
+    stopifnot( !is.null( nbar ) )
+   
     # ------------------------------#
     # Generate school and district IDs
     # ------------------------------#
-    # generates vector of school and district assignments, assuming equal sizes of everything
+    # generates vector of school and district IDs, 
+    # assuming equal sizes of everything
     if ( is.null( S.id ) ) {
-        assignments <- gen_assignments( J, K, nbar )
+        assignments <- gen_cluster_ids( nbar, J, K )
         # N-length vector of indiv school assignments i.e. (1,1,2,2,3,3)
         S.id        <- assignments[['S.id']] 
         # N-length vector of indiv district assignments i.e. (1,1,1,2,2,2)
@@ -145,14 +215,16 @@ gen_base_sim_data <- function(dgp.params.list) {
         Sigma.w.full   <- gen_RE_cov_matrix( Sigma.w0, Sigma.w1, Sigma.w )
         
         # generate full vector of district random effects and impacts
-        w01.k <- matrix(mvtnorm::rmvnorm(K, mean = rep(0, 2*M), 
-                                         sigma = Sigma.w.full), nrow = K, ncol = 2*M)
+        w01.k <- matrix(mvtnorm::rmvnorm(K, mean = rep(0, 2 * M), 
+                                         sigma = Sigma.w.full), 
+                        nrow = K, ncol = 2 * M)
         w0.k  <- w01.k[,1:M, drop = FALSE]
-        w1.k  <- w01.k[,(M+1):(2*M), drop = FALSE]
+        w1.k  <- w01.k[,(M + 1):(2 * M), drop = FALSE]
         return(list(V.k = V.k, w0.k = w0.k, w1.k = w1.k))
     }
     
     if ( has.level.three ) {
+        stopifnot( !is.null( K ) )
         level.three.data <- gen.level.three.data(K, M, rho.V, 
                                                  eta0.sq, eta1.sq, 
                                                  rho.w0, rho.w1, kappa.w) 
@@ -195,7 +267,7 @@ gen_base_sim_data <- function(dgp.params.list) {
             nrow = J*K, ncol = 2*M
         )
         u0.jk  <- u01.jk[,1:M, drop = FALSE]
-        u1.jk  <- u01.jk[,(M+1):(2*M), drop = FALSE]
+        u1.jk  <- u01.jk[,(M + 1):(2 * M), drop = FALSE]
         
         return(list(X.jk = X.jk, u0.jk = u0.jk, u1.jk = u1.jk))
     }
@@ -243,7 +315,7 @@ gen_base_sim_data <- function(dgp.params.list) {
     Xi1.ijk <- matrix(Xi1, nrow = N, ncol = M, byrow = TRUE) 
     
     # loop through each individual student
-    for(i in 1:N)
+    for (i in 1:N)
     {
         if ( has.level.three )
         {
@@ -287,46 +359,65 @@ gen_base_sim_data <- function(dgp.params.list) {
     
     ID <- data.frame( S.id = S.id, D.id = D.id )
     
+    res = NA
     if ( has.level.three ) {
-        return(list(Y0 = Y0.ijk, Y1 = Y1.ijk, 
+        res <- list(Y0 = Y0.ijk, Y1 = Y1.ijk, 
                     V.k = V.ijk, X.jk = X.ijk, 
-                    C.ijk = C.ijk, ID = ID ))
+                    C.ijk = C.ijk, ID = ID )
     } else {
         ID$D.id <- NULL
-        return(list(Y0 = Y0.ijk, Y1 = Y1.ijk, 
+        res <- list(Y0 = Y0.ijk, Y1 = Y1.ijk, 
                     V.k = NULL, X.jk = X.ijk, 
-                    C.ijk = C.ijk, ID = ID ))
+                    C.ijk = C.ijk, ID = ID )
+    }
+    
+    if ( return.as.dataframe ) {
+        res <- makelist_samp(res)
+        if ( no.list && length(res) == 1 ) {
+            return( res[[1]] )
+        } else {
+            return( res )
+        }
+    } else {
+        return( res )
     }
 }
 
+
+
+
+
 #' @title Generate simulated multi-level data (simulation function)
-#' 
-#' @description Generates simulated data for multi-level
-#' RCTs for pump-suppored designs and models for
-#' both unobserved and observed potential outcomes.
-#' 
-#' Takes in two options:
-#' - a pumpresult object
-#' OR
-#' - a list of necessary data-generating parameters
-#' - the context (d_m)
-#' - Tbar (proportion assigned to treatment)
-#' 
-#' This function is beyond the main scope of calculating power,
-#' and is instead used for simulating data.
-#' For more info on use, see the simulation vignette.
-#' 
-#' @param pump.object A pumpresult object.
-#' @param d_m string; a single context, which is a design and model code. 
-#' See pump_info() for list of choices.
-#' @param model.params.list list; model parameters such as ICC,
-#' R2, etc. See simulation vignette for details.
-#' @param Tbar scalar; the proportion of samples 
-#' that are assigned to the treatment.
 #'
-#' @return list; potential outcomes, covariates,
-#' observed outcomes, and treatment assignment.
-#' 
+#' @description Generates simulated data for multi-level RCTs for
+#'   pump-suppored designs and models for both unobserved and observed
+#'   potential outcomes.
+#'
+#'   Takes in two options: - a pumpresult object OR - a list of
+#'   necessary data-generating parameters - the context (d_m) - Tbar
+#'   (proportion assigned to treatment)
+#'
+#'   This function is beyond the main scope of calculating power, and
+#'   is instead used for simulating data. For more info on use, see
+#'   the simulation vignette.
+#'
+#' @param pump.object A pumpresult object.
+#' @param d_m string; a single context, which is a design and model
+#'   code. See pump_info() for list of choices.
+#' @param param.list list; model parameters such as ICC, R2, etc. See
+#'   simulation vignette for details.
+#' @param Tbar scalar; the proportion of samples that are assigned to
+#'   the treatment.
+#' @param return.as.dataframe TRUE means return list of dataframes,
+#'   one for each outcome.  FALSE means return components of the
+#'   covariates, etc., in a list.
+#' @param no.list Only relevant if return.as.dataframe=TRUE.
+#'   no.list=TRUE means if M=1 return the dataframe, not a list of
+#'   length 1.  FALSE means return a list of length 1, even if there
+#'   is only 1 outcome.
+#' @return list; potential outcomes, covariates, observed outcomes,
+#'   and treatment assignment.
+#'
 #'
 #' @export
 #' @examples
@@ -347,39 +438,21 @@ gen_base_sim_data <- function(dgp.params.list) {
 #'                   tnum = 200
 #' )
 #' sim.data <- gen_sim_data(pump.object = pp)
-#'
+#' 
 gen_sim_data <- function(
-    d_m = NULL, model.params.list = NULL, Tbar = 0.5,
-    pump.object = NULL)
-{
+        d_m = NULL, param.list = NULL, Tbar = 0.5,
+        pump.object = NULL,
+        return.as.dataframe = TRUE,
+        no.list = TRUE ) {
     
-    # If first thing is a pump.object, swap
-    if ( !is.null(d_m) && (is.pumpresult(d_m) || is.pumpgridresult(d_m)) ) {
-        pump.object <- d_m
-        d_m<- NULL
-    }
+    param.list = process_and_generate_param_list( d_m = d_m,
+                                                  param.list = param.list,
+                                                  Tbar = Tbar,
+                                                  pump.object = pump.object )
+    d_m = param.list$d_m
+    Tbar = param.list$Tbar
     
-    if(is.null(pump.object))
-    {
-      if(is.null(d_m) | is.null(model.params.list))
-      {
-          stop("You must provide either a pump object
-                or both a design string (d_m) and list of model params.")
-      }
-    } else {
-      if(!is.null(d_m) | !is.null(model.params.list))
-      {
-          stop("You must provide either a pump object or
-                a design string (d_m) and list of model params pair (not both).")
-      }
-      model.params.list <- params(pump.object)
-      d_m <- d_m(pump.object)
-      Tbar <- model.params.list$Tbar
-      model.params.list$rho.default <- model.params.list$rho
-    }
-    
-    dgp.params.list <- convert_params(model.params.list)
-    sim.data <- gen_base_sim_data(dgp.params.list)
+    sim.data <- gen_base_sim_data( param.list, return.as.dataframe = FALSE )
     sim.data$T.x <- gen_T.x(
         d_m = d_m,
         S.id = sim.data$ID$S.id,
@@ -388,7 +461,16 @@ gen_sim_data <- function(
     )
     sim.data$Yobs <- gen_Yobs(sim.data, T.x = sim.data$T.x)
     
-    return(sim.data)
+    if ( return.as.dataframe ) {
+        res <- makelist_samp(sim.data)
+        if ( no.list && length(res) == 1 ) {
+            return( res[[1]] )
+        } else {
+            return( res )
+        }
+    } else {
+        return(sim.data)
+    }
 }
 
 
@@ -396,99 +478,104 @@ gen_sim_data <- function(
 #' @title Converts model params into DGP params (simulation function)
 #'
 #' @description Converts user-provided parameters such as ICC and
-#'   omega into data-generating parameters that can produce simulated
-#'   data, such as variance values and covariate coefficients.
+#'   omega into data-generating parameters for the multilevel random
+#'   effects model used to produce simulated data, such as variance
+#'   values and covariate coefficients.
 #'
 #'   This function is beyond the main scope of calculating power, and
 #'   is instead used for simulating data. For more info on use, see
 #'   the simulation vignette.
 #'
-#' @param model.params.list list; model parameters such as ICC, R2,
-#'   etc.
+#' @param param.list list; model parameters such as ICC, R2, etc.
 #'
 #' @return list; data-generating parameters.
 #'
 #' @export
-convert_params <- function(model.params.list) {
+convert_params <- function(param.list) {
     
-    # save out useful paramters
-    M <- model.params.list[['M']]
-    ICC.2 <- model.params.list[['ICC.2']]
-    ICC.3 <- model.params.list[['ICC.3']]
-    R2.1 <- model.params.list[['R2.1']]
-    R2.2 <- model.params.list[['R2.2']]
-    R2.3 <- model.params.list[['R2.3']]
-    omega.2 <- model.params.list[['omega.2']]
-    omega.3 <- model.params.list[['omega.3']]
-    rho.default <- model.params.list[['rho.default']]
+    # save out useful parameters
+    M <- param.list[['M']]
+    ICC.2 <- param.list[['ICC.2']]
+    ICC.3 <- param.list[['ICC.3']]
+    R2.1 <- param.list[['R2.1']]
+    R2.2 <- param.list[['R2.2']]
+    R2.3 <- param.list[['R2.3']]
+    omega.2 <- param.list[['omega.2']]
+    omega.3 <- param.list[['omega.3']]
+    rho.default <- param.list[['rho.default']]
+    
+    # If only single outcome, rho not needed.  Give default so code runs.
+    if ( M == 1 && is.null( rho.default ) ) {
+        rho.default = 0
+    }
     
     # If no district info, set district parameters to 0
     has.level.three <- TRUE
-    if ( is.null( ICC.3 ) ) {
+    if ( is.null( param.list$K ) ) {
         has.level.three <- FALSE
-        ICC.3 <- model.params.list$ICC.3 <- rep(0, M)
-        R2.3 <- model.params.list$R2.3 <- rep(0, M)
-        omega.3 <- model.params.list$omega.3 <- rep(0, M)
-        K <- model.params.list$K <- 1
+        ICC.3 <- param.list$ICC.3 <- rep(0, M)
+        R2.3 <- param.list$R2.3 <- rep(0, M)
+        omega.3 <- param.list$omega.3 <- rep(0, M)
+        K <- param.list$K <- 1
     }
     
-    if(is.null(rho.default) & any(c(
-       is.null(model.params.list$rho.V), 
-       is.null(model.params.list$rho.w0),
-       is.null(model.params.list$rho.w1),
-       is.null(model.params.list$rho.X), 
-       is.null(model.params.list$rho.u0),
-       is.null(model.params.list$rho.u1),
-       is.null(model.params.list$rho.C), 
-       is.null(model.params.list$rho.r))))
+    if (is.null(rho.default) & any(c(
+        is.null(param.list$rho.V), 
+        is.null(param.list$rho.w0),
+        is.null(param.list$rho.w1),
+        is.null(param.list$rho.X), 
+        is.null(param.list$rho.u0),
+        is.null(param.list$rho.u1),
+        is.null(param.list$rho.C), 
+        is.null(param.list$rho.r))))
     {
-        stop('Please provide either a rho.default or
-             ALL necessary correlation matrices.')
+        stop(paste('Please provide either a rho.default',
+                   'or ALL necessary correlation matrices.'))
     }
     
     default.rho.matrix <- gen_corr_matrix(M = M, rho.scalar = rho.default)
     
-    if(is.null(model.params.list$rho.V))
+    if (is.null(param.list$rho.V))
     {
-        model.params.list$rho.V  <- default.rho.matrix 
+        param.list$rho.V  <- default.rho.matrix 
     }
-    if(is.null(model.params.list$rho.w0))
+    if (is.null(param.list$rho.w0))
     {
-        model.params.list$rho.w0  <- default.rho.matrix 
+        param.list$rho.w0  <- default.rho.matrix 
     }
-    if(is.null(model.params.list$rho.w1))
+    if (is.null(param.list$rho.w1))
     {
-        model.params.list$rho.w1 <- default.rho.matrix 
+        param.list$rho.w1 <- default.rho.matrix 
     }
     
-    if(is.null(model.params.list$rho.X))
+    if (is.null(param.list$rho.X))
     {
-        model.params.list$rho.X  <- default.rho.matrix 
+        param.list$rho.X  <- default.rho.matrix 
     }
-    if(is.null(model.params.list$rho.u0))
+    if (is.null(param.list$rho.u0))
     {
-           model.params.list$rho.u0  <- default.rho.matrix 
+        param.list$rho.u0  <- default.rho.matrix 
     }
-    if(is.null(model.params.list$rho.u1))
+    if (is.null(param.list$rho.u1))
     {
-        model.params.list$rho.u1 <- default.rho.matrix 
+        param.list$rho.u1 <- default.rho.matrix 
     }
     
-    if(is.null(model.params.list$rho.C))
+    if (is.null(param.list$rho.C))
     {
-        model.params.list$rho.C  <- default.rho.matrix 
+        param.list$rho.C  <- default.rho.matrix 
     }
-    if(is.null(model.params.list$rho.r))
+    if (is.null(param.list$rho.r))
     {
-           model.params.list$rho.r  <- default.rho.matrix 
+        param.list$rho.r  <- default.rho.matrix 
     }
     
     convert.scalar <- function(x, M)
     {
-        if( is.null(x) )
+        if ( is.null(x) )
         {
             return( rep(0, M) )
-        } else if( length(x) == 1 )
+        } else if ( length(x) == 1 )
         {
             return( rep(x, M) )
         } else
@@ -496,7 +583,7 @@ convert_params <- function(model.params.list) {
             return(x)
         }
     }
-    model.params.list$MDES <- convert.scalar(model.params.list$MDES, M)
+    param.list$MDES <- convert.scalar(param.list$MDES, M)
     R2.1    <- convert.scalar(R2.1, M)
     R2.2    <- convert.scalar(R2.2, M)
     R2.3    <- convert.scalar(R2.3, M)
@@ -506,31 +593,31 @@ convert_params <- function(model.params.list) {
     omega.3 <- convert.scalar(omega.3, M)
     
     # defaults for kappa matrices
-    if(is.null(model.params.list[['kappa.w']]))
+    if (is.null(param.list[['kappa.w']]))
     {
-        model.params.list$kappa.w <- matrix(0, M, M) 
+        param.list$kappa.w <- matrix(0, M, M) 
     }
-    if(is.null(model.params.list[['kappa.u']]))
+    if (is.null(param.list[['kappa.u']]))
     {
-        model.params.list$kappa.u <- matrix(0, M, M) 
+        param.list$kappa.u <- matrix(0, M, M) 
     }
     
     # default for grand mean
-    if(is.null(model.params.list[['Xi0']]))
+    if (is.null(param.list[['Xi0']]))
     {
-        model.params.list$Xi0 <- 0
+        param.list$Xi0 <- 0
     }
-
+    
     # check ICC is valid
-    if( any( ICC.2 + ICC.3 >= 1 ) )
+    if ( any( ICC.2 + ICC.3 >= 1 ) )
     {
         stop(paste('ICC.2 + ICC.3 must be less than 1.
                    ICC.2:', ICC.2, 'ICC3:', ICC.3))
     }
     
     # random intercepts variances
-    tau0.sq <- ( (1 - R2.2) / (1- R2.1) ) * ( ICC.2 / (1 - ICC.2 - ICC.3) )
-    eta0.sq <- ( (1 - R2.3) / (1- R2.1) ) * ( ICC.3 / (1 - ICC.2 - ICC.3) )
+    tau0.sq <- ( (1 - R2.2) / (1 - R2.1) ) * ( ICC.2 / (1 - ICC.2 - ICC.3) )
+    eta0.sq <- ( (1 - R2.3) / (1 - R2.1) ) * ( ICC.3 / (1 - ICC.2 - ICC.3) )
     # covariate coefficients
     delta   <- sqrt( (R2.2 / (1 - R2.1)) *  ( ICC.2 / (1 - ICC.2 - ICC.3) ) )
     xi      <- sqrt( (R2.3 / (1 - R2.1)) *  ( ICC.3 / (1 - ICC.2 - ICC.3) ) )
@@ -541,54 +628,54 @@ convert_params <- function(model.params.list) {
     eta1.sq <- omega.3 * (eta0.sq + xi^2)
     
     # grand mean impact
-    Xi1 <- model.params.list[['MDES']] *
+    Xi1 <- param.list[['MDES']] *
         sqrt(xi^2 + gamma^2 + delta^2 + eta0.sq + tau0.sq + 1)
     
-    dgp.params.list <- list(
+    new.param.list <- list(
         has.level.three = has.level.three
-        , M = model.params.list[['M']]                    # number of outcomes
-        , J = model.params.list[['J']]                    # number of schools
-        , K = model.params.list[['K']]                    # number of districts
-        , nbar = model.params.list[['nbar']]              # number of individuals per school
-        , Xi0 = model.params.list[['Xi0']]                # scalar grand mean outcome under no treatment
-        , Xi1 = Xi1                                       # scalar grand mean impact
+        , M = param.list[['M']]                    # number of outcomes
+        , J = param.list[['J']]                    # number of schools
+        , K = param.list[['K']]                    # number of districts
+        , nbar = param.list[['nbar']]              # number of individuals per school
+        , Xi0 = param.list[['Xi0']]                # scalar grand mean outcome under no treatment
+        , Xi1 = Xi1                                # scalar grand mean impact
     )
     
     if ( has.level.three ) {
-        dgp.params.list <- c( dgp.params.list, list( 
+        new.param.list <- c( new.param.list, list( 
             # -------------------------------------------- level 3
-            xi = xi                                          # M-vector of coefficient of district covariates
-            , rho.V = model.params.list[['rho.V']]           # MxM correlation matrix of district covariates
-            , eta0.sq = eta0.sq                              # M-vector of variances of district random effects
-            , eta1.sq = eta1.sq                              # M-vector of variances of district impacts
-            , rho.w0 = model.params.list[['rho.w0']]         # MxM matrix of correlations for district random effects
-            , rho.w1 = model.params.list[['rho.w1']]         # MxM matrix of correlations for district impacts
-            , kappa.w = model.params.list[['kappa.w']]       # MxM matrix of correlations between district random effects and impacts
+            xi = xi                             # M-vector of coefficient of district covariates
+            , rho.V = param.list[['rho.V']]     # MxM correlation matrix of district covariates
+            , eta0.sq = eta0.sq                 # M-vector of variances of district random effects
+            , eta1.sq = eta1.sq                 # M-vector of variances of district impacts
+            , rho.w0 = param.list[['rho.w0']]   # MxM matrix of correlations for district random effects
+            , rho.w1 = param.list[['rho.w1']]   # MxM matrix of correlations for district impacts
+            , kappa.w = param.list[['kappa.w']] # MxM matrix of correlations between district random effects and impacts
         ) )
     }
     
-    dgp.params.list <- c( dgp.params.list, list(
+    new.param.list <- c( new.param.list, list(
         # -------------------------------------------- level 2
-        delta = delta                                    # M-vector of coefficients of school covariates
-        , rho.X = model.params.list[['rho.X']]           # MxM correlation matrix of school covariates
-        , tau0.sq = tau0.sq                              # M-vector of variances of school random effects
-        , tau1.sq = tau1.sq                              # M-vector of variances of school impacts
-        , rho.u0 = model.params.list[['rho.u0']]         # MxM matrix of correlations for school random effects
-        , rho.u1 = model.params.list[['rho.u1']]         # MxM matrix of correlations for school impacts
-        , kappa.u = model.params.list[['kappa.u']]       # MxM matrix of correlations between school random effects and impacts
+        delta = delta                             # M-vector of coefficients of school covariates
+        , rho.X = param.list[['rho.X']]           # MxM correlation matrix of school covariates
+        , tau0.sq = tau0.sq                       # M-vector of variances of school random effects
+        , tau1.sq = tau1.sq                       # M-vector of variances of school impacts
+        , rho.u0 = param.list[['rho.u0']]         # MxM matrix of correlations for school random effects
+        , rho.u1 = param.list[['rho.u1']]         # MxM matrix of correlations for school impacts
+        , kappa.u = param.list[['kappa.u']]       # MxM matrix of correlations between school random effects and impacts
         # -------------------------------------------- level 1
-        , gamma = gamma                                  # M-vector of coefficients of individual covariates
-        , rho.C = model.params.list[['rho.C']]           # MxM correlation matrix of individual covariates
-        , rho.r = model.params.list[['rho.r']]           # MxM matrix of correlations for individual residuals
+        , gamma = gamma                           # M-vector of coefficients of individual covariates
+        , rho.C = param.list[['rho.C']]           # MxM correlation matrix of individual covariates
+        , rho.r = param.list[['rho.r']]           # MxM matrix of correlations for individual residuals
     ) )
     
-    nulls <- vapply( dgp.params.list, is.null, logical(1) )
-    dgp.params.list <- dgp.params.list[ !nulls ]
-    
-    return(dgp.params.list)
+    nulls <- vapply( new.param.list, is.null, logical(1) )
+    new.param.list <- new.param.list[ !nulls ]
     
+    return(new.param.list)
 }
 
+
 #' @title Generates school and district assignments (simulation
 #'   function)
 #'
@@ -608,7 +695,7 @@ convert_params <- function(model.params.list) {
 #'   individual.
 #'
 #' @export
-gen_assignments <- function(J, K, nbar){
+gen_cluster_ids <- function(nbar, J, K){
     
     J <- ifelse(is.null(J), 1, J)
     K <- ifelse(is.null(K), 1, K)
@@ -618,7 +705,7 @@ gen_assignments <- function(J, K, nbar){
     S.id <- rep(NA, N)
     start.index <- 1
     end.index <- nbar
-    for(j in 1:(K*J))
+    for (j in 1:(K*J))
     {
         S.id[start.index:end.index] <- j
         start.index <- end.index + 1
@@ -629,7 +716,7 @@ gen_assignments <- function(J, K, nbar){
     start.index <- 1
     n.k <- N/K
     end.index <- n.k
-    for(k in 1:K)
+    for (k in 1:K)
     {
         D.id[start.index:end.index] <- k
         start.index <- end.index + 1
@@ -667,16 +754,18 @@ gen_T.x <- function(d_m, S.id, D.id, Tbar)
     d_m <- strsplit(d_m, "_")
     d_m <- d_m[[1]][[1]]
     
-    if(d_m == "d1.1") {
+    if (d_m == "d1.1") {
         T.x <- randomizr::simple_ra(N = length(S.id), prob = Tbar)
-    } else if( d_m == "d2.1" || d_m == "d3.1" ) {
+    } else if ( d_m == "d2.1" || d_m == "d3.1" ) {
         T.x <- randomizr::block_ra( S.id, prob = Tbar )
-    } else if( d_m == "d2.2" ) { 
+    } else if ( d_m == "d2.2" ) { 
         T.x <- randomizr::cluster_ra( S.id, prob = Tbar )
-    } else if( d_m == "d3.3" ) {
+    } else if ( d_m == "d3.3" ) {
         T.x <- randomizr::cluster_ra( D.id, prob = Tbar )
-    } else if( d_m == "d3.2" ) {
-        T.x <- randomizr::block_and_cluster_ra( blocks = D.id, clusters = S.id, prob = Tbar )
+    } else if ( d_m == "d3.2" ) {
+        T.x <- randomizr::block_and_cluster_ra( 
+            blocks = D.id, clusters = S.id, prob = Tbar 
+        )
     } else {
         stop(print(paste('design', d_m, 'not implemented yet')))
     }
@@ -717,20 +806,32 @@ gen_Yobs <- function(full.data, T.x) {
 #' @param T.x vector of treatment assignments
 #' @return List of dataframes.
 #' @keywords internal
-makelist_samp <-function(samp.obs, T.x) {
+makelist_samp <- function(samp.obs, T.x = NULL ) {
     
-    mdat.rn <- rep( NULL, ncol(samp.obs$Yobs) )
-    for(m in 1:ncol(samp.obs$Yobs))
+    if ( is.null( T.x ) ) {
+        T.x = samp.obs$T.x
+    }
+    
+    M = ncol( samp.obs$C.ijk )
+    
+    tx_assigned = TRUE
+    if ( is.null( samp.obs[['Yobs']] ) ) {
+        stopifnot( is.null( T.x ) )
+        tx_assigned = FALSE
+    } else {
+        stopifnot( !is.null( T.x ) )
+    }
+    
+    mdat.rn <- rep( NULL, M )
+    for (m in 1:M)
     {
         # level 3
-        if(!is.null(samp.obs[['V.k']]))
+        if (!is.null(samp.obs[['V.k']]))
         {
             mdat.rn[[m]] <- data.frame(
-                Yobs        = samp.obs[['Yobs']][,m],
                 V.k         = samp.obs[['V.k']][,m],
                 X.jk        = samp.obs[['X.jk']][,m],
                 C.ijk       = samp.obs[['C.ijk']][,m],
-                T.x         = T.x,
                 S.id        = as.factor(samp.obs$ID$S.id),
                 D.id        = as.factor(samp.obs$ID$D.id)
             )
@@ -738,13 +839,16 @@ makelist_samp <-function(samp.obs, T.x) {
             # level 2
         {
             mdat.rn[[m]] <- data.frame(
-                Yobs        = samp.obs[['Yobs']][,m],
                 X.jk        = samp.obs[['X.jk']][,m],
                 C.ijk       = samp.obs[['C.ijk']][,m],
-                T.x         = T.x,
                 S.id        = as.factor(samp.obs$ID$S.id)
             )
         }
+        
+        if ( tx_assigned ) {
+            mdat.rn[[m]]$Yobs = samp.obs[['Yobs']][,m]
+            mdat.rn[[m]]$T.x         = T.x
+        }
     }
     return(mdat.rn)
 }
